// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

var ClientRequest = require('../util/apim-request');
var ServerError = require('../util/server-error');
var Util = require('../util/util');
var _ = require('lodash');
var cliConfig = require('apiconnect-config').loadConfig();
var g = require('strong-globalize')();
var logger = require('apiconnect-cli-logger');

function Catalogs() {
  this.catalogsList = {};
}

/**
 * Verify that environment name <tt>envName</tt> is valid for the provider org that the users is logged in
 *
 * @param envName  name of environment
 * @param callback {Function (err, env)}
 *                 callback function that is called once the env has been verified; it is invoked with 2 parameters:
 *                 err - if an error occured
 *                 env - contains details of the environment as it is returned from the apim stack
 */
Catalogs.prototype.verifyEnvName = function(envName, orgName, callback) {
  var self = this;
  self.list(function(err, orgs) {
    if (err) {
      return callback(err);
    }

    var orgOptions = {
      organization: orgName,
      allCatalogs: true,
    };

    var envs = self.filter(orgOptions)[orgName];
    if (envs) {
      for (var i = 0; i < envs.length; i++) {
        if (envName === envs[i].shortName) {
          return callback(null, envs[i]);
        }
      }
      return callback(new Error(g.f(
        'The catalog is invalid. Valid catalogs are [%s].',
        _.map(envs, 'shortName'))));
    }
    callback(new Error(g.f('The catalog is invalid.')));
  });
};

/**
 *
 * List all catalogs associated to a user.
 *
 * @param cb {Function(err, catalogs)} callback function. The format of the catalogs object:
 *          <code>
 *            [
 *               { displayName: "", environments: [], name: ""},
 *            ]
 *          </code>
 */
Catalogs.prototype.list = function(cb) {
  var self = this;
  var opts = Util.requestDefaultOptions();
  opts.url += allCatalogsUrl();

  ClientRequest.get(opts, function(err, rsp, body) {
    if (err) {
      logger.error(err);
      return cb(err);
    }

    if (rsp.statusCode === 200) {
      self.catalogsList = JSON.parse(body);
      return cb(null, self.catalogsList);
    }
    else {
      err = new ServerError(rsp.statusCode, body);
      logger.error(err);
      return cb(err);
    }

  });
};
/**
 * This should be invoked after {@see #list} method has been invoked. It will filter the results and return
 * the following object:
 * <code>
 *     [
 *        "<orgName>": [ {shortName: <shortNameEnv>, name: <name>, id: <id>]
 *     ]
 * </code>
 *
 *  The logic is:
 *  if --all-organizations is true then:
 *     - if --all-catalogs true fetch all catalogs
 *     - if --catalog return only all the catalogs that matches that name across all organizations
 *
 * if --all-catalogs is true then
 *    - return all catalogs within the set organization
 * if --organization then
 *    - if --all-catalogs return all catalogs within than organization
 *    - if --catalog return catalog within the organization (or error if not found)
 *
 * if --catalog then
 *    return catalog within set organization
 *
 *
 * @param opts {object} contains the following properties
 *    <code>
 *        {
 *           allOrganizations: <boolean>,
 *           allCatalogs: <boolean>,
 *           organization: <orgName>,
 *           catalog: <catalogName>
  *        }
 *    </code>
 */
Catalogs.prototype.filter = function(opts) {
  var self = this;
  var rsp = {};
  var orgRsp;

  if (opts.allOrganizations) { //go get all organizations  and then all catalogs
    self.catalogsList.forEach(function(org) {
      rsp[org.name] = [];
      org.environments.forEach(function(catalog) {
        if (opts.allCatalogs || (opts.catalog && catalog.shortName === opts.catalog)) {
          var c = JSON.parse(JSON.stringify(catalog));
          delete c.permissions;
          c.fullId = catalogFullId({
            server: opts.server,
            org: org.name,
            catalog: c.shortName });
          rsp[org.name].push(c);
        }
      });
    });
  }
  else if (opts.allCatalogs) {
    orgRsp = _.find(self.catalogsList, { name: opts.organization });
    if (orgRsp) {
      rsp[orgRsp.name] = [];
      orgRsp.environments.forEach(function(catalog) {
        var c = JSON.parse(JSON.stringify(catalog));
        delete c.permissions;
        c.fullId = catalogFullId({
          server: opts.server,
          org: orgRsp.name,
          catalog: c.shortName });
        rsp[orgRsp.name].push(c);
      });
    }
  }
  else if (opts.organization) {
    orgRsp = _.find(self.catalogsList, { name: opts.organization });
    if (orgRsp) {
      rsp[orgRsp.name] = [];
      var catalogRsp = _.find(orgRsp.environments, { shortName: opts.catalog });
      if (catalogRsp) {
        var c = JSON.parse(JSON.stringify(catalogRsp));
        delete c.permissions;
        c.fullId = catalogFullId({
          server: opts.server,
          org: orgRsp.name,
          catalog: c.shortName });
        rsp[orgRsp.name].push(c);
      }
    }
  }
  else {
    orgRsp = _.find(self.catalogsList, { name: opts.organization });
    if (orgRsp) {
      rsp[orgRsp.name] = [];
      orgRsp.environments.forEach(function(catalog) {
        var c = JSON.parse(JSON.stringify(catalog));
        delete c.permissions;
        c.fullId = catalogFullId({
          server: opts.server,
          org: orgRsp.name,
          catalog: c.shortName });
        rsp[orgRsp.name].push(c);
      });
    }
  }
  return rsp;
};

module.exports = Catalogs;

function allCatalogsUrl() {
  return '/v1/me/permissions';
};

function catalogFullId(opts) {
  var uriOpts = {
    protocol: 'apic-catalog',
    hostname: opts.server,
  };
  return cliConfig.toUri(uriOpts) + '/orgs/' + opts.org + '/catalogs/' + opts.catalog;
};
