// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

var Entity = require('./entity');
var ObjectID = require('bson-objectid');
var _ = require('lodash');
var g = require('strong-globalize')();
var logger = require('apiconnect-cli-logger');
var request = require('../util/apim-request');
var u = require('../util/util');

/**
 * Default constructor for an api entity - extends the {Entity} class.
 *
 * @param entity  {Entity}
 * @constructor
 */
function Api(entity) {
  this.entity = entity;
  _.extend(this, entity);
}

Api.prototype.dump = function() {
  logger.info(this.getContent());
};
/**
 * Convert to apic format - converts swagger to the apic meta-data model.
 *
 * @param orgName name or id of the org provider
 * @param callback {Function(err, object)}
 * @param options {object} [optional] support one options expandPolicy: true - expands policies within the tasks
 */
Api.prototype.convertToApim = function(orgName, callback, options) {
  var apiSwaggerContent = this.getContent();
  var self = this;
  var opts = u.requestDefaultOptions();
  opts.headers['content-type'] = 'application/vnd.ibm-apim.swagger2+json';
  if (apiSwaggerContent.basePath && apiSwaggerContent.basePath.indexOf('/') !== 0) {
    apiSwaggerContent.basePath = '/' + apiSwaggerContent.basePath;
  }
  options = options || {};
  opts.url += previewApiUrl(orgName, apiSwaggerContent.basePath || '/', options.preview);
  opts.json = apiSwaggerContent;
  opts.method = 'POST';
  try {
    request.post(opts.url, opts, function(err, httpRes, body) {
      if (err) {
        callback(err, null);
      }
      else if (httpRes.statusCode < 200 || httpRes.statusCode > 399) {
        var error = new Error(u.stringifyServerError(body), httpRes.statusCode);
        error.body = body;
        callback(error, null);
      }
      else {
        if (options && options.expandPolicy) {
          //expand policies
          body.api.name = body.api.info['x-ibm-name'];
          body.api._class = 'com.ibm.apimgmt.config.impl.data.webapi.ApiDefinition';
          if (body.resources) {
            for (var i = 0; i < body.resources.length; i++) {
              body.resources[i]._class = 'com.ibm.apimgmt.config.impl.data.webapi.ApiResourceDefinition';
            }
          }
          postProcessing(self, body.api);
        }
        callback(null, body);
      }
    });
  }
  catch (error) {
    callback(error, null);
  }

};
/**
 * Push the api swagger(yaml or json) to the apimanager stack
 *
 * @param options {object} contains various options
 * @param callback {Function(err,object)} callbac function that gets invoked when the operation is completed.
 */
Api.prototype.push = function(options, callback) {
  options = options || {};
  options.expandPolicy = false;
  options.preview = false;
  this.convertToApim(u.providerOrgName(), callback, options);
};

Api.prototype.pull = function(options, callback) {
  throw new Error(g.f('Not implemented!'));
};

/**
 * Validate entity
 *
 * @param opts {object} - options: serverValidation: true|false - if true then send the request to the server
 * @param cb {Function(err, result)}
 */
Api.prototype.validate = function(opts, cb) {
  if (opts && (opts.serverValidation === 'true')) {
    //server validation
    var orgName = u.providerOrgName();
    this.convertToApim(orgName, cb);
  }
  else {
    //calling super
    //offline validation
    Entity.prototype.validate.call(this, opts, cb);
  }

};

var postProcessing = function(self, api) {
  if (api && api.interfaces && api.interfaces.length > 0) {
    for (var i = 0; i < api.interfaces.length; i++) {
      if (api.interfaces[i].assembly && api.interfaces[i].assembly.tasks) {
        //process tasks
        for (var j = 0; j < api.interfaces[i].assembly.tasks.length; j++) {
          var task = api.interfaces[i].assembly.tasks[j];
          var proxyPolicy = createProxyPolicy(self, api);
          api.policies = [];
          api.policies.push(proxyPolicy);
          for (var k = 0; k < task.policies.length; k++) {
            task.policies = [];
            task.policies.push(proxyPolicy);
          }
        }
      }
    }
  }
};

/**
 * Create proxy policy - this eventually will be removed once x-ibm-configuration defines the assembly accordingly
 *
 * @param self
 * @param api
 */
var createProxyPolicy = function(self, api) {
  var swaggerApi = self.getContent();
  var scheme = swaggerApi.schemes && swaggerApi.schemes.length === 1 ? swaggerApi.schemes[0] : 'https';
  var proxy = {
    name: ObjectID.generate(),
    policyType: 'apim.proxy',
    policyTypeVersion: 1,
    properties: {
      match: swaggerApi.basePath,
      targetUrl: scheme + '://' + swaggerApi.host,
      sslProfile: '',
    },
    displayName: 'PROXY',
  };
  return proxy;
};

var previewApiUrl = function(orgName, basePath, preview) {
  return '/v1/apimanager/orgs/' + orgName + '/apis?' + (preview ? 'preview=true' : '') + '&basePath=' + basePath;
};

module.exports = Api;
