// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

var Validator = require('./validator');
var debug = require('debug')('apiconnect-cli-util:entity');
var fs = require('fs');
var jsYaml = require('js-yaml');
var u = require('../util/util');

debug('Loaded...');
/**
 * Represents a project entity that points to a file within the project
 */

function Entity(/**String*/path) {
  this.path = path;
}

/**
 * the absolute path ot the entity
 */
Entity.prototype.path = function() {
  return this.path;
};


/**
 * Return the name of the entity or the basename associated to this file.
 *
 */
Entity.prototype.name = function() {

  if (!this.content) {
    this.read();
  }

  this.content = this.content || {};
  this.content.info = this.content.info || {};

  if (this.entityContentType === 'api') {
    if (!this.content.info['x-ibm-name'] && this.content.info.title) {
      // If the api doesn't have the x-ibm-name let's add it for staging and in memory only
      this.content.info['x-ibm-name'] = u.generateSlugFromText(this.content.info.title);
    }
    return this.content.info['x-ibm-name'];
  }
  else {
    return this.content.info.title;
  }
};

/**
 * Return the info associated to this entity
 */
Entity.prototype.info = function() {
  if (!this.content) {
    this.read();
  }
  if (this.content && this.content.info) {
    return this.content.info;
  }
};
/**
 * Read the content of the entity synchronously.
 *
 * @return {object} return an object that represents the read content
 */
Entity.prototype.read = function() {
  this.error = null;
  try {
    this.content = jsYaml.safeLoad(fs.readFileSync(this.path));
    if (typeof this.content === 'string') {
      this.entityContentType = 'unknown';
      this.error = 'Unsupported Content';
      this.content = {};
      return this.content;
    }
    if (this.content.swagger) {
      this.entityContentType = 'api';
    }
    else if (this.content.product) {
      this.entityContentType = 'product';
    }
    else {
      this.entityContentType = 'unknown';
    }
    return this.content;
  }
  catch (error) {
    debug(error);
    this.error = error;
    this.entityContentType = 'unknown';
    return null;
  }
};

Entity.prototype.getContent = function() {
  if (!this.content) {
    this.read();
  }
  return this.content;
};
/**
 * Validate the entity and return if the entry is valid or not.
 *
 * @param opts {object}
 * @param cb {Function(err, status)}  - status is a json that indicate if entry is valid {"valid" : true}
 */
Entity.prototype.validate = function(opts, cb) {
  if (this.error) {
    cb(this.error, null);
  }
  else {
    var validator = Validator.createValidator(this.entityType());
    if (validator) {
      var status = validator.validate(this.getContent());
      if (status.valid) {
        cb(null, status);
      }
      else {
        cb(status.error, status);
      }
    }
    else {
      cb(null, { valid: true });
    }
  }
};


/**
 * The content type associated to this entity (api, product or policy?)
 * @return {String} valid values: 'api', 'product', 'policy?', 'unknown' (the unknown entry should be discarded
 */
Entity.prototype.entityType = function() {
  if (!this.entityContentType) {
    this.read();
  }
  return this.entityContentType;
};

/**
 * Save the <tt>data</data> and associated to this entity
 *
 * @param path save the data to the location represented by the path
 * @param cb {Function(err)} invoked when save operation has finished; if an err occured, it will be pass as the argument of the function, otherwise it will be null
 */
Entity.prototype.save = function(path, cb) {
  var callback = cb;

  fs.writeFile(path, jsYaml.safeDump(this.content), function(err) {
    return callback && callback(err);
  });
};


module.exports = Entity;


