// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

var NodeRSA = require('node-rsa');
var Promise = require('bluebird');
var _ = require('lodash');
var cliConfig = require('apiconnect-config');
var cliUtil = require('../util/util');
var debug = require('debug')('apiconnect-cli-util:provider-apps');
var fs = Promise.promisifyAll(require('fs'));
var g = require('strong-globalize')();
var logger = require('apiconnect-cli-logger');
var orgs = require('../orgs/organizations');
var path = require('path');
var request = require('request-promise');

function ProviderApp() {
  this.runtime = '';
  this.bluemix = null;
  this.liberty = null;
}

/**
 * Create a new provider app
 *
 * @param {object} options
 * @param {string} options.app application name
 * @param {string} options.server APIm server
 * @param {string} options.org APIm organization
 * @param {string} options.runtime application runtime. `liberty` or `bluemix`.
 * @param {string} [options.title] application title. Defaults to clugified version of name.
 * @param {string} [options.libertyClusterName] collective name if application runs on liberty collective.
 *
 * @return {ProviderApp}
 */
function create(options) {
  options = options || {};
  options.app = options.app || cliUtil.generateSlugFromText(options.title);
  var uriPath = '/v1/orgs/' + options.org + '/provider-apps';
  var data = {
    title: options.title || options.name,
    name: options.app,
  };
  var supportedRuntime = [ 'Bluemix', 'Liberty' ];
  switch (options.runtime) {
    case 'bluemix':
      data.bluemix = {};
      break;
    case 'liberty':
      data.liberty = {
        collective: options.libertyClusterName,
      };
      break;
    default:
      throw new Error(g.f('The %s runtime is invalid.   Valid runtime values are %s',
        options.runtime, supportedRuntime.join(', ')));
  }

  return _request(options.server, uriPath, 'post', data).then(function(data) {
    return _parseResult(data, options.org);
  });
}
exports.create = create;

/**
 * @param {object} [options]
 * @param {string} options.app application name
 * @param {string} options.server APIm server
 * @param {string} options.org APIm organization
 * @param {object} options.data Data to update on application
 */
function update(options) {
  var uriPath = '/v1/orgs/' + options.org + '/provider-apps/' + options.app;

  return _request(options.server, uriPath, 'put', options.data).then(function(data) {
    return _parseResult(data, options.org);
  });
}
exports.update = update;

/**
 * @param {object} options
 * @param {string} org APIm organization
 * @param {string} server APIm server
 *
 * @return {Array.<ProviderApp>}
 */
function list(options) {
  var uriPath = '/v1/orgs/' + options.org + '/provider-apps';
  return _request(options.server, uriPath, 'get').then(function(data) {
    return _.map(data, function(entry) {
      var app = _parseResult(entry, options.org);

      var uriOpts = {
        protocol: 'apic-app',
        hostname: options.server,
      };
      app.fullId = cliConfig.loadConfig().toUri(uriOpts) + '/orgs/' + options.org + '/apps/' + app.name;

      return app;
    });
  });
}
exports.list = list;

/**
 * @param {object} options
 * @param {string} app app name
 * @param {string} org APIm organization
 * @param {string} server APIm server
 *
 * @return {Array.<ProviderApp>}
 */
function listAll(options) {
  if (options.org) {
    return list(options);
  }
  return orgs.listOrgsAsync(options.server).then(function(organizations) {
    return Promise.all(Promise.map(organizations, function(org) {
      return list({
        server: options.server,
        org: org.name,
      });
    })).then(function(apps) {
      return _.flatten(apps);
    });
  });
}
exports.listAll = listAll;

/**
 * @param {object} options
 * @param {string} app app name
 * @param {string} org APIm organization
 * @param {string} server APIm server
 * @param {boolean} [getEncrypted] get encrypted content
 *
 * @return {ProviderApp}
 */
function get(options) {
  var uriPath = '/v1/orgs/' + options.org + '/provider-apps/' + options.app;

  if (options.getEncrypted) {
    return _loadKeys().then(function(key) {
      var publicKey = key.exportKey('public');
      return _request(options.server, uriPath + '/encrypt', 'post', { publicKey: publicKey }).then(function(data) {
        return _parseResult(data, options.org, key);
      });
    });
  } else {
    return _request(options.server, uriPath, 'get').then(function(data) {
      return _parseResult(data, options.org);
    });
  }
}
exports.get = get;

function _parseResult(data, orgName, key) {
  if (!data) {
    var err = new Error(g.f('The provider app %s is not found.', app));
    err.status = 404;
    throw err;
  }
  var app = new ProviderApp();
  app.name = data.name;
  app.title = data.title;
  app.id = data.id;
  app.orgName = orgName;
  if (data.liberty && data.liberty.collective) {
    app.runtime = 'liberty';
    app.liberty = {
      collectiveName: data.liberty.collective.name,
      collectiveId: data.liberty.collective.id,
      applicationName: data.name,
      applicationId: data.id,

      hostHeader: data.name + '.' + orgName,
    };

    if (data.liberty.collective.controllers && data.liberty.collective.controllers.length >= 1) {
      app.liberty.username = data.liberty.collective.controllers[0].username;
      app.liberty.password = data.liberty.collective.controllers[0].password;
      app.liberty.controllerHosts = _.map(data.liberty.collective.controllers, function(c) {
        return _.pick(c, [ 'host', 'port' ]);
      });

      if (key) {
        app.liberty.password = key.decrypt(app.liberty.password, 'utf8');
      }
    }
  } else if (data.bluemix && data.bluemix.organization) {
    app.runtime = 'bluemix';
    app.bluemix = data.bluemix;
  }

  return app;
}

exports.testKeyPath = null;
function _loadKeys() {
  var configDir = exports.testKeyPath || cliConfig.getUserConfigDir();
  var cliPrivateKeyPath = path.join(configDir, 'private.pem');
  var cliPublicKeyPath = path.join(configDir, 'public.pem');
  var key = new NodeRSA(null, null, { encryptionScheme: 'pkcs1' });
  return fs.statAsync(cliPrivateKeyPath).then(function loadKeys() {
    debug('loadKeys');
    return fs.readFileAsync(cliPrivateKeyPath, 'utf8').then(function(data) {
      debug('importing private key');
      key.importKey(data);
    }).then(function() {
      return fs.readFileAsync(cliPublicKeyPath, 'utf8');
    }).then(function(data) {
      debug('importing public key');
      key.importKey(data);
    }).then(function() {
      debug('return crypto');
      return key;
    });
  }).catch(function makeKeys() {
    debug('makeKeys');
    logger.writeln(g.f('Creating keys...this may take some time'));
    key.generateKeyPair();
    var privateKey = key.exportKey('private');
    var publicKey = key.exportKey('public');
    debug('writing private key');
    return fs.writeFileAsync(cliPrivateKeyPath, privateKey, 'utf8').then(function() {
      debug('writing public key');
      return fs.writeFileAsync(cliPublicKeyPath, publicKey, 'utf8');
    }).then(function() {
      debug('return crypto');
      return key;
    });
  });
}

function _request(server, path, method, data) {
  method = method || 'get';
  var opts = cliUtil.requestDefaultOptions(server);

  opts.url += path;
  opts.method = method;
  opts.json = true;
  if (data) {
    opts.body = data;
  }

  return request(opts)
    .catch(function(err) {
      if (err.error.errors && err.error.errors.length > 0) {
        var errMsgs = _.values(err.error.errors[0]);
        var newErr = new Error(_.first(errMsgs));
        newErr.errors = errMsgs;
        newErr.status = err.statusCode || 500;
        throw newErr;
      }
      throw new Error(g.f('The {{--server}} and {{--organization}} options are required.'));
    });
}
