// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

/**
 * A wrapper around 'request' module
 */

var debug = require('debug')('apiconnect-cli-util:request');
var g = require('strong-globalize')();
var logger = require('apiconnect-cli-logger');
var request = require('request');

var log = function(uri, method) {
  try {
    if (typeof uri === 'string') {
      logger.info(g.f('Requesting: %s %s', (method || 'GET'), uri));
      debug('Requesting: ' + (method || 'GET ') + uri);
    }
    else {
      logger.info(g.f('Requesting: %s %s', (method || uri.method), uri.url));
      debug('Requesting: ' +
        (method || uri.method) + ' ' + uri.url);
      if (uri.json) {
        logger.info(g.f('Body: %s' + JSON.stringify(uri.json, null, ' ')));
        debug('Body: ' + JSON.stringify(uri.json, null, ' '));
      }
    }
  }
  catch (error) {
    logger.error(error);
  }
};
/**
 * Issue a get request
 *
 * @param uri {String|Object}
 * @param options  {object}
 * @param callback {Function (err, res, body)}
 * @returns {*}
 */
var get = function(uri, options, callback) {
  log(uri, 'GET');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.get(uri, function(err, res, body) {
      // request header.encoding is set to null when we are not sure if the body coming back is
      // xml format or zip format instead of json obj (for scenario: downloading wsdl definition for SOAP API)
      // If request header.encoding is set to null, then convert it back to string if there is an error.
      if (!err && (res.statusCode < 200 || res.statusCode > 399)) {
        if (uri.encoding === null) {
          body = JSON.parse(body.toString('utf8'));
        }
      }
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.get(uri, options, function(err, res, body) {
    // if there is error, then if body is of type buffer convert it back to json object
    if (res.statusCode < 200 || res.statusCode > 399) {
      if (uri.encoding === null) {
        body = JSON.parse(body.toString('utf8'));
      }
    }
    printResponse(err, res, body);
    cb(err, res, body);
  });
};

var post = function(uri, options, callback) {
  log(uri, 'POST');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.post(uri, function(err, res, body) {
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.post(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    cb(err, res, body);
  });

};

var del = function(uri, options, callback) {
  log(uri, 'DELETE');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.del(uri, function(err, res, body) {
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.del(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    cb(err, res, body);
  });

};

var put = function(uri, options, callback) {
  log(uri, 'PUT');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.put(uri, function(err, res, body) {
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.put(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    cb(err, res, body);
  });

};


/**
 *
 * @param uri
 * @param options
 * @param callback
 * @returns {*}
 */
var patch = function(uri, options, callback) {
  log(uri, 'PATCH');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.patch(uri, function(err, res, body) {
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.path(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    cb(err, res, body);
  });
};

var head = function(uri, options, callback) {
  log(uri, 'HEAD');
  var cb = callback;
  if (typeof options === 'function') {
    cb = options;
    return request.head(uri, function(err, res, body) {
      printResponse(err, res, body);
      cb(err, res, body);
    });
  }
  return request.head(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    cb(err, res, body);
  });

};

var jar = function() {
  return request.jar();
};

var cookie = function(/*string*/cookie) {
  return request.cookie(cookie);
};

var printResponse = function(err, res, body) {
  if (res) {
    logger.info(g.f('status code: %s', res.statusCode));
    debug('status code: ' + res.statusCode);
  }
  if (err) {
    logger.info(g.f('Error: %s', err));
    debug('error: ' + err);
  }
  if (body) {
    logger.info(g.f('Body: %s' + body));
    debug('Body: ' + body);
  }
};

/**
 * Issue a get request
 *
 * @param uri
 * @param options
 * @param callback
 * @returns {*}
 */
module.exports = function(uri, options, callback) {
  return request(uri, options, function(err, res, body) {
    printResponse(err, res, body);
    callback(err, res, body);
  });
};

module.exports.get = get;
module.exports.put = put;
module.exports.del = del;
module.exports.post = post;
module.exports.patch = patch;
module.exports.head = head;
module.exports.jar = jar;
module.exports.cookies = cookie;
